from enum import Enum
from pathlib import Path
from typing import Annotated

import tomlkit
from rich import print
from rich.progress import Progress, SpinnerColumn, TextColumn
from typer import Argument, Option, Typer

from casoar.main import (
    NoConfigFoundError,
    TooManyConfigFilesError,
    read_config,
)
from casoar.models import Hooks
from casoar.writer import write_hooks

app = Typer()


class ConfigFileName(str, Enum):
    PYPROJECT = "pyproject.toml"
    CUSTOM = ".casoar.toml"


@app.command()
def install():
    try:
        config = read_config()
        write_hooks(config, hooks_folder=".git/hooks")
    except NoConfigFoundError:
        print("[red]Error: No configuration found for [b]casoar [/red]")
        print(
            "[yellow]Hint:[/yellow] Run [green]casoar init[/green] to create a configuration file"
        )
    except TooManyConfigFilesError:
        print(
            "[red]Error: Both .casoar.yml and pyproject.toml have configuration for casoar[/red]"
        )
        print(
            "[yellow]Hint:[/yellow] Remove one of the configuration files to fix this error"
        )


@app.command()
def init(
    config_file: Annotated[ConfigFileName, Argument()] = ConfigFileName.PYPROJECT.value,
    force: Annotated[bool, Option("--force")] = False,
) -> None:
    pyproject_path = Path(ConfigFileName.PYPROJECT.value)
    custom_config_path = Path(ConfigFileName.CUSTOM.value)

    if custom_config_path.exists():
        if not force:
            print(
                f"[red]Error: Configuration file {custom_config_path} already exists. To bypass this error, use '--force' option[/red]"
            )
            raise SystemExit(1)

        else:
            custom_config_path.unlink()

    pyproject_data = {}
    if pyproject_path.exists():
        pyproject_data = tomlkit.loads(pyproject_path.read_text())

    if "tool" in pyproject_data and "casoar" in pyproject_data["tool"]:
        if not force:
            print(
                "[red]Error: Configuration for casoar already exists in pyproject.toml[/red]"
            )
            raise SystemExit(1)
        else:
            del pyproject_data["tool"]["casoar"]
            with open(pyproject_path, "wt") as file:
                tomlkit.dump(pyproject_data, file)

    config = Hooks.get_sample()

    match config_file:
        case ConfigFileName.PYPROJECT:
            if not pyproject_path.exists():
                with Progress(
                    SpinnerColumn(),
                    TextColumn("[progress.description]{task.description}"),
                ) as progress:
                    task = progress.add_task(
                        description="Creating pyproject.toml file...", total=1
                    )
                    pyproject_path.touch()
                    progress.update(
                        task, completed=1, description="Creating pyproject.toml file ✅"
                    )

            if "tool" not in pyproject_data:
                pyproject_data["tool"] = tomlkit.table()

            if "casoar" in pyproject_data["tool"] and not force:
                print(
                    f"[red]Error: Configuration for casoar already exists in {config_file.value}[/red]"
                )
                raise SystemExit(1)
            sample = Hooks.get_sample()
            pyproject_data["tool"]["casoar"] = tomlkit.table()
            for k, v in sample.model_dump(by_alias=True, exclude_unset=True).items():
                pyproject_data["tool"]["casoar"][k] = v

            # pyproject_data.get("tool")["casoar"].add("virtual_env", ".venv")

            with open(pyproject_path, "wt") as file:
                tomlkit.dump(pyproject_data, file)
        case ConfigFileName.CUSTOM:
            with open(custom_config_path, "w") as file:
                tomlkit.dump(config.model_dump(exclude_unset=True, by_alias=True), file)


if __name__ == "__main__":
    app()



from pathlib import Path
from typing import Generator, Tuple

from pydantic import BaseModel, ConfigDict, Field, field_serializer

from .command import Command


class Hooks(BaseModel):
    virtual_env: Path | None = None
    pre_commit: list[Command] = Field(alias="pre-commit", default_factory=list)
    commit_msg: list[Command] = Field(alias="commit-msg", default_factory=list)
    pre_push: list[Command] = Field(alias="pre-push", default_factory=list)

    model_config = ConfigDict(populate_by_name=True)

    @field_serializer("virtual_env", return_type=str)
    def serialize_virtualenv(virtual_env) -> str:
        return str(virtual_env)

    def __iter__(self) -> Generator[Tuple[str, list[Command]], None, None]:
        for hook_type, infos in self.model_fields.items():
            if hook_type != "virtual_env":
                yield infos.alias or hook_type, getattr(self, hook_type)

    @classmethod
    def get_sample(cls) -> "Hooks":
        return cls(
            virtual_env=Path(".venv"),
            pre_commit=[
                Command(
                    command='ruff format --check --exclude="**/.venv/*"',
                    message="Please format your code",
                ),
                Command(
                    command='ruff check --exclude="**/.venv/*"',
                    message="Please lint your code",
                ),
            ],
            commit_msg=[
                Command(
                    command="cz check --commit-msg-file $1",
                    message="Please check your commit message",
                ),
            ],
        )



from pathlib import Path

import tomlkit

from casoar.models import Hooks


class NoConfigFoundError(Exception):
    pass


class TooManyConfigFilesError(Exception):
    pass


def read_config() -> Hooks:
    config_file = Path("./.casoar.toml")
    pyproject_file = Path("./pyproject.toml")
    pyproject_data = tomlkit.loads(pyproject_file.read_text())

    if "tool" not in pyproject_data.keys():
        pyproject_data.add("casoar")

    d = pyproject_data.get("tool")
    if d is None:
        raise NoConfigFoundError("No configuration found for casoar")

    if "casoar" in d.keys():
        if config_file.exists():
            raise TooManyConfigFilesError(
                "Both .casoar.toml and pyproject.toml have configuration for casoar"
            )
        return Hooks.model_validate(pyproject_data["tool"]["casoar"])

    if config_file.exists():
        return Hooks.model_validate(tomlkit.loads(config_file.read_text()))

    raise NoConfigFoundError("No configuration found for casoar")



import stat
from pathlib import Path

from rich.progress import Progress, SpinnerColumn, TextColumn

from casoar.models import Hooks


def write_hooks(hooks: Hooks, hooks_folder: Path | str = Path("./git/hooks")) -> None:
    if isinstance(hooks_folder, str):
        hooks_folder = Path(hooks_folder)
    if not hooks_folder.exists():
        hooks_folder.mkdir(parents=True)

    for hook_type, commands in hooks:
        if len(commands) == 0:
            continue

        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
        ) as progress:
            task = progress.add_task(description=f"Writing {hook_type} hook", total=1)

            hook_path = hooks_folder / hook_type

            file_content = (
                "#!/bin/bash\n"
                "# This file was automatically generated by captain-hook\n"
            )
            if hooks.virtual_env is not None:
                file_content += f"source {hooks.virtual_env}/bin/activate\n"

            for command in commands:
                file_content += f"{command.to_bash_string()}\n"

            hook_path.write_text(file_content)

            hook_path.chmod(hook_path.stat().st_mode | stat.S_IEXEC)

            progress.update(
                task, completed=1, description=f"Writing {hook_type} hook ✅"
            )



from pydantic import BaseModel


class Command(BaseModel):
    command: str
    silent: bool = False
    message: str | None = None

    def to_bash_string(self) -> str:
        if self.silent and self.message is not None:
            suffix = f"|| {{ echo '{self.message}' ; exit 1; }}" if self.message else ""
            return f"{self.command} >/dev/null 2>&1 {suffix}"

        if self.silent:
            return f"{self.command} >/dev/null 2>&1"

        if self.message is not None:
            return (
                f"command_output=$({self.command} 2>&1)\n"
                "command_exit_code=$?\n"
                "if [ $command_exit_code -ne 0 ]; then\n"
                f"    echo '{self.message}'\n"
                '    echo "$command_output"\n'
                "    exit 1\n"
                "fi"
            )

        return self.command
